<?php

namespace Wappointment\Services;

use Wappointment\Models\Appointment as AppointmentModel;
use Wappointment\Models\Client;
use Wappointment\Helpers\Events;
use Wappointment\WP\Helpers as WPHelpers;
class Appointment
{
    public static function delete($id)
    {
        $status = \Wappointment\Models\Appointment::destroy($id);
        if ($status) {
            (new \Wappointment\Services\Availability())->regenerate();
        }
        return $status;
    }
    public static function create($data)
    {
        return \Wappointment\Models\Appointment::create($data);
    }
    public static function confirm($id)
    {
        $appointment = \Wappointment\Models\Appointment::with('client')->where('id', $id)->where('status', \Wappointment\Models\Appointment::STATUS_AWAITING_CONFIRMATION)->first();
        if (empty($appointment)) {
            throw new \WappointmentException("Can't find appointment", 1);
        } else {
            $result = $appointment->update(['status' => \Wappointment\Models\Appointment::STATUS_CONFIRMED]);
            if ($result) {
                \Wappointment\Helpers\Events::dispatch('AppointmentConfirmedEvent', ['appointment' => $appointment, 'client' => $appointment->client]);
            }
            return $result;
        }
    }
    public static function patch($id, $data)
    {
        $result = \Wappointment\Models\Appointment::where('id', $id)->update($data);
        if ($result) {
            (new \Wappointment\Services\Availability())->regenerate();
        }
        return $result;
    }
    public static function unixToDb($unixTS)
    {
        return \Wappointment\ClassConnect\Carbon::createFromTimestamp($unixTS)->toDateTimeString();
    }
    public static function tryBook(\Wappointment\Models\Client $client, $start_at, $end_at, $type, $service)
    {
        if (self::canBook($start_at, $end_at)) {
            return self::processBook($client, $start_at, $end_at, $type, $service);
        }
    }
    public static function adminBook(\Wappointment\Models\Client $client, $start_at, $end_at, $type, $service)
    {
        if (self::canBook($start_at, $end_at, true)) {
            return self::processBook($client, $start_at, $end_at, $type, $service, true);
        }
    }
    private static function processBook(\Wappointment\Models\Client $client, $start_at, $end_at, $type, $service, $forceConfirmed = false)
    {
        $start_at = self::unixToDb($start_at);
        $end_at = self::unixToDb($end_at);
        $appointmentData = ['start_at' => $start_at, 'end_at' => $end_at, 'type' => $type, 'client_id' => $client->id, 'edit_key' => \md5($client->id . $start_at), 'status' => $forceConfirmed ? \Wappointment\Models\Appointment::STATUS_CONFIRMED : self::getDefaultStatus($service)];
        return self::book($appointmentData, $client, $forceConfirmed);
    }
    private static function canBook($start_at, $end_at, $is_admin = false)
    {
        //test that this is not a double booking scenario
        $start_at_str = self::unixToDb($start_at);
        $end_at_str = self::unixToDb($end_at);
        if (\Wappointment\Models\Appointment::where('status', '>=', \Wappointment\Models\Appointment::STATUS_AWAITING_CONFIRMATION)->where(function ($query) use($start_at_str, $end_at_str) {
            $query->where(function ($query) use($start_at_str, $end_at_str) {
                $query->where('start_at', $start_at_str);
                $query->where('end_at', $end_at_str);
            });
            /**
             *     DB:  □□□□□□□□□□□■■■■■■■■■■■■■■■■■■■■■■■■□□□□□□□□□□
             *     this: □□□□□□□□□□□□□□□□□□■■■■■■■■■■■□□□□□□□□□□□□□□□□
             *     Db contains this
             */
            $query->orWhere(function ($query) use($start_at_str, $end_at_str) {
                $query->where('start_at', '<=', $start_at_str);
                $query->where('end_at', '>=', $end_at_str);
            });
            /*
             *     DB:   □□□□□□□□□□□□□□□□□□■■■■■■■■■■■□□□□□□□□□□□□□□□□
             *     This: □□□□□□□□□□□■■■■■■■■■■■■■■■■■■■■■■■■□□□□□□□□□□
             *      this contains DB
             */
            $query->orWhere(function ($query) use($start_at_str, $end_at_str) {
                $query->where('start_at', '>=', $start_at_str);
                $query->where('end_at', '<=', $end_at_str);
            });
            /*
             *     DB:   □□□□□□□□□□□■■■■■■■■■■■■■□□□□□□□□□□□□□□□□□□□□□
             *     This: □□□□□□□□□□□□□□□□□□■■■■■■■■■■■□□□□□□□□□□□□□□□□
             *     DB intersects this
             */
            $query->orWhere(function ($query) use($start_at_str, $end_at_str) {
                $query->where('start_at', '<', $start_at_str);
                $query->where('end_at', '>', $start_at_str);
            });
            /*
             *     DB:   □□□□□□□□□□□□□□□□□□■■■■■■■■■■■□□□□□□□□□□□□□□□□
             *     This: □□□□□□□□□□□■■■■■■■■■■■■■□□□□□□□□□□□□□□□□□□□□□
             */
            $query->orWhere(function ($query) use($start_at_str, $end_at_str) {
                $query->where('start_at', '<', $end_at_str);
                $query->where('end_at', '>', $end_at_str);
            });
        })->exists()) {
            throw new \WappointmentException('Slot already booked', 1);
        }
        if ($is_admin === true) {
            return true;
        }
        //staff being booked
        $activeStaff = \Wappointment\Services\Settings::get('activeStaffId');
        //test that were in the booking availability
        if ((new \Wappointment\Services\Availability())->isAvailable($start_at, $end_at, $activeStaff)) {
            return true;
        }
        throw new \WappointmentException('Slot not available', 1);
    }
    private static function book($data, \Wappointment\Models\Client $client, $is_admin = false)
    {
        $appointment = self::create($data);
        if ($appointment->status == \Wappointment\Models\Appointment::STATUS_AWAITING_CONFIRMATION) {
            \Wappointment\Helpers\Events::dispatch('AppointmentBookedEvent', ['appointment' => $appointment, 'client' => $client]);
        } else {
            \Wappointment\Helpers\Events::dispatch('AppointmentConfirmedEvent', ['appointment' => $appointment, 'client' => $client]);
        }
        //return $appointment;
        //if availability has not been refreshed for a while we refresh it now otherwise we queue a job for it
        if (!\defined('DISABLE_WP_CRON') || $is_admin === true) {
            //when web cron is disabled we need an immediate refresh of availability
            (new \Wappointment\Services\Availability())->regenerate();
        } else {
            if ((int) \Wappointment\WP\Helpers::getStaffOption('since_last_refresh') > 2) {
                (new \Wappointment\Services\Availability())->regenerate();
            } else {
                $count_since_last_refresh = (int) \Wappointment\WP\Helpers::getStaffOption('since_last_refresh', false, 0) + 1;
                \Wappointment\WP\Helpers::setStaffOption('since_last_refresh', $count_since_last_refresh);
                \Wappointment\Services\Queue::tryPush('Wappointment\\Jobs\\AvailabilityRegenerate', ['staff_id' => \Wappointment\Services\Settings::get('activeStaffId')], 'availability');
            }
            //we immediately spawn a process to trigger availability regenerate in the back
            spawn_cron();
        }
        return $appointment;
    }
    public static function reschedule($edit_key, $start_at)
    {
        if (!(bool) \Wappointment\Services\Settings::get('allow_rescheduling')) {
            throw new \WappointmentException('Appointment rescheduling is not allowed', 1);
        }
        $appointment = \Wappointment\Models\Appointment::where('edit_key', $edit_key)->first();
        $oldAppointment = $appointment->replicate();
        if (empty($appointment)) {
            throw new \WappointmentException("Can't find appointment", 1);
        }
        if (!$appointment->canStillReschedule()) {
            throw new \WappointmentException("Can't reschedule appointment anymore", 1);
        }
        $result = $appointment->update(['start_at' => self::unixToDb($start_at), 'end_at' => self::unixToDb($start_at + $appointment->getDurationInSec())]);
        if ($result) {
            (new \Wappointment\Services\Availability())->regenerate();
            \Wappointment\Helpers\Events::dispatch('AppointmentRescheduledEvent', ['appointment' => $appointment, 'client' => $appointment->client()->first(), 'oldAppointment' => $oldAppointment]);
        }
        return $result;
    }
    public static function tryCancel($edit_key)
    {
        if (!(bool) \Wappointment\Services\Settings::get('allow_cancellation')) {
            throw new \WappointmentException('Appointment cancellation is not allowed', 1);
        }
        $appointment = \Wappointment\Models\Appointment::where('edit_key', $edit_key)->where('status', '>=', \Wappointment\Models\Appointment::STATUS_AWAITING_CONFIRMATION)->first();
        if (empty($appointment)) {
            throw new \WappointmentException("Can't find appointment", 1);
        }
        if (!$appointment->canStillCancel()) {
            throw new \WappointmentException("Can't cancel appointment anymore", 1);
        }
        self::cancel($appointment);
    }
    public static function cancel(\Wappointment\Models\Appointment $appointment)
    {
        \Wappointment\Models\Log::canceledAppointment($appointment);
        $client = $appointment->client()->first();
        $staff_id_regenerate = $appointment->getStaffId();
        $result = $appointment->destroy($appointment->id);
        if ($result) {
            (new \Wappointment\Services\Availability())->regenerate($staff_id_regenerate);
            \Wappointment\Helpers\Events::dispatch('AppointmentCanceledEvent', ['appointment' => $appointment, 'client' => $client]);
        }
        return $result;
    }
    private static function getDefaultStatus($service)
    {
        $default_status = (int) \Wappointment\Services\Settings::get('approval_mode') === 1 ? \Wappointment\Models\Appointment::STATUS_CONFIRMED : \Wappointment\Models\Appointment::STATUS_AWAITING_CONFIRMATION;
        return apply_filters('wappointment_appointment_default_status', $default_status, $service);
    }
}
